@@ -17,6 +17,6 @@ G_BEGIN_DECLS
 
 GUdevDevice      *fbd_dev_led_get_device  (FbdDevLed *led);
 void              fbd_dev_led_set_max_brightness (FbdDevLed *led, guint max_brightness);
-void              fbd_dev_led_set_color (FbdDevLed *led, FbdFeedbackLedColor color);
+void              fbd_dev_led_set_supported_color (FbdDevLed *led, FbdFeedbackLedColor color);
 
 G_END_DECLS
