@@ -13,6 +13,8 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (FbdFeedbackSound, fbd_feedback_sound, FBD, FEEDBACK_SOUND, FbdFeedbackBase);
 
-const char *fbd_feedback_sound_get_effect (FbdFeedbackSound *self);
+FbdFeedbackSound *fbd_feedback_sound_new_from_file_name (const char *filename);
+const char       *fbd_feedback_sound_get_effect (FbdFeedbackSound *self);
+const char       *fbd_feedback_sound_get_file_name (FbdFeedbackSound *self);
 
 G_END_DECLS
